#include "sim900.h"


extern SIM900_struct sim900struct;
extern volatile uint16_t eraser_cnt;

void sim900_pin_config(void)
{
	//Reset
	DDRD |= (1<<5);
	PORTD &= (~(1<<5));
	
	//Power_on
	DDRD |= (1<<6);
	PORTD &= (~(1<<6));
	
	//Status Input PIN
	DDRD &= (~(1<<1));
	PORTD |= ((1<<1));
	
	//DTR (out)
	DDRB |= (1<<4);
	PORTB &= (~(1<<4));
	
}


void sim900_reset(void)
{
	// Modu reaguje na negative reset 
	// Tutaj positiver reset, poniewa zastosowano tranzystor NPN
	// zwierajcy do masy pin resetu moduu
	PORTD |= ((1<<5)); 
	delay_ms(1);
	PORTD &= (~(1<<5));
	wdt_reset();
	delay_ms(500);
	wdt_reset();
	delay_ms(500);
	wdt_reset();
	delay_ms(500);
	wdt_reset();
	delay_ms(500);
}

bool sim900_power_switch(bool mode)
{
	if (mode == 0) // Wycz
	{
		PORTD |= ((1<<6));
		while (SIM900_STATUS_PIN == 1) {wdt_reset();}
		wdt_reset();
		delay_ms(500);
		wdt_reset();
		delay_ms(500);
		PORTD &= (~(1<<6));
	} 
	else // Zacz
	{
		PORTD |= ((1<<6));
		wdt_reset();
		delay_ms(500);	
		wdt_reset();
		delay_ms(500);
		PORTD &= (~(1<<6));
		wdt_reset();
		delay_ms(500);
		wdt_reset();
		delay_ms(500);
		wdt_reset();
		delay_ms(500);
		wdt_reset();
		delay_ms(500);
		wdt_reset();
		delay_ms(500);
		wdt_reset();
		delay_ms(500);										
		
		if (SIM900_STATUS_PIN)
			return 1;
		else
			return 0;
	}	
}

bool sim900_autobaud(void)
{
	uint8_t error_counter = 10;
	uint8_t tmp_cnt;
	sim900struct.last_recv_chr=0;

	while(sim900struct.last_recv_chr != 'A')
	{
		sim900_char('A');
	
		for (tmp_cnt=0;tmp_cnt<5;tmp_cnt++)
		{
			sim900struct.last_recv_chr=usart_gsm_receive_nowait();
			
			if (sim900struct.last_recv_chr == 'A')
				break;
			
			delay_ms(5);
			wdt_reset();
		
		}

		error_counter--;
		
		if (error_counter==0)
			return 0;
	}
	return 1;
}


bool sim900_configure(bool pre_config)
{
	delay_ms(100);
	
	if (pre_config)
		SIM900_send_command("ATE0",SIM900_OK_ANS,5);        //Wyczenie Echa
	else
	{		
		SIM900_send_command("AT+CLIP=1",SIM900_OK_ANS,5);   //Identyfikacja numeru dzwonicego
		SIM900_send_command("AT+CRC=1",SIM900_OK_ANS,5);    //Rozszerzona informacja o numerze dzwonicego
		SIM900_send_command("AT+CNMI=2,2",SIM900_OK_ANS,5); //Wywietlanie wiadomoci sms po otrzymaniu bez zapisywania
		SIM900_send_command("AT+CMGF=1",SIM900_OK_ANS,5);   //SMS text mode
	}		

	return 1;
}


bool sim900_gpio_exec(bool mode, uint8_t gpio_number, uint8_t function, bool state)
{
	bool ans=0;
	uint8_t cnt=5;
	
	rprintfInit(usart_gsm_char);
		
	while (ans != 1)
	{
		wdt_reset();
		sim900struct.buffer_element=0;
		sim900struct.recv_buffer[0]=0;
		rprintf("AT+SGPIO=%d,%d,%d,%d\r",mode,gpio_number,function,state);
		
		ans = wait_ans(SIM900_OK_ANS,300);	
		
		cnt--;
		if (cnt== 0) return 0;
	}
	
	sim900struct.opcje.out_state[gpio_number-1]= state;

	return 1;
}




bool SIM900_send_command(uint8_t * command, uint8_t * wait_answ, uint8_t max_retr)
{
	do 
	{
		wdt_reset();
		sim900struct.buffer_element=0;
		sim900struct.recv_buffer[0]=0;
		sim900_string(command); 
		sim900_char('\r'); 
		max_retr--;
		
		if (max_retr == 0) return 0;       
	} while (wait_ans(wait_answ,200)==0);
	

	debug_txt(COMMAND_SEND_OK);
	delay_ms(10);
	debug_txt(command);
	wdt_reset();
	delay_ms(100);
	return 1;
}



bool wait_ans( uint8_t * ans, uint16_t time_ms)
{	
	while(time_ms--)
	{
		wdt_reset();
		if (strstr(sim900struct.recv_buffer,"ERROR") != 0)
			return 0;
		
		if (strstr(sim900struct.recv_buffer,ans) != 0)
		{	
			return 1;
		}		

		delay_ms(1);
	}
	
	return 0;	
}

sim_state sim900_check_pin()
{
	uint16_t ms;
	
	sim900struct.buffer_element=0;
	sim900struct.recv_buffer[0]=0;
	sim900_string("AT+CPIN?\r");
	
	ms=0;
	
	while (ms < 3000)
	{	
		wdt_reset();
		delay_ms(1);
		ms++;
		
		if (strstr(sim900struct.recv_buffer,"READY") != NULL)
			return READY;
		if (strstr(sim900struct.recv_buffer,"SIM PIN") != NULL)
			return PIN_ENTER;
		if (strstr(sim900struct.recv_buffer,"SIM PIN2") != NULL)
			return PIN2_ENTER;
		if (strstr(sim900struct.recv_buffer,"SIM PUK") != NULL)
			return PUK_ENTER;
		if (strstr(sim900struct.recv_buffer,"SIM PUK") != NULL)
			return PUK2_ENTER;
		if (strstr(sim900struct.recv_buffer,"SIM not inserted") != NULL)
			return SIM_NOT_INSERTED;					
	}
	
	return ERROR;
}

bool sim900_enter_pin(uint8_t * pin_number)
{
	uint16_t ms;
	bool ans;
	
	sim900struct.buffer_element=0;
	sim900struct.recv_buffer[0]=0;
	
	sim900_string("AT+CPIN=");
	sim900_string(pin_number);
	sim900_string("\r");
	
	ans = wait_ans(SIM900_OK_ANS,1000);
	wdt_reset();
	delay_ms(300);
	wdt_reset();
	return ans;
}

void sim900_call_ans(void)
{
	sim900_string("ATA\r");	
}

void sim900_call_refuse(void)
{
	sim900_string("ATH\r");
}

bool sim900_send_sms(uint8_t * number, uint8_t * msg)
{
	bool ans;
	
	sim900_string("AT+CMGS=\"");
	sim900_string(number);
	sim900_string("\"");
	sim900_char(0x0D);
	wdt_reset();
	delay_ms(50);
	sim900_string(msg);
	wdt_reset();
	delay_ms(200);
	sim900struct.buffer_element=0;
	sim900struct.recv_buffer[0]=0;
	sim900_char(0x1A);
	wdt_reset();
	ans = wait_ans(SIM900_OK_ANS,8000); //dugi timeout 8s
	return ans;
}

bool delete_all_sms()
{
	bool ans=0;
	wdt_reset();
	sim900struct.buffer_element=0;
	sim900struct.recv_buffer[0]=0;
	sim900_string("AT+CMGDA=\"DEL ALL\"\r");
	ans = wait_ans(SIM900_OK_ANS,3000); //dugi timeout 8s
	
	return ans;
}


void SIM900_buffer_erase(void)
{
	uint8_t i;
	
	for (i=0;i<200;i++)
	{
		sim900struct.recv_buffer[i]=0;
		wdt_reset();
	}		
		
	sim900struct.last_recv_chr=0;
	sim900struct.buffer_element=0;
}


ISR(SIG_USART0_RECV)
{
    volatile uint8_t znak;
	led2_on;
	eraser_cnt=0;
	znak=UDR0;
	
	sim900struct.last_recv_chr=znak;
	
 	if (sim900struct.buffer_element < 198)
	{
		sim900struct.recv_buffer[sim900struct.buffer_element++]=znak;
		sim900struct.recv_buffer[sim900struct.buffer_element]=0; 	
	} 
	else
	{
		sim900struct.buffer_element=0;
		sim900struct.recv_buffer[0]=0;
	}		
}