#ifndef SIM900_H_
#define SIM900_H_

#include <stdbool.h>
#include <avr/io.h>
#include <stdint.h>
#define F_CPU 8000000ul
#include <util/delay.h>
#include <string.h>
#include <avr/interrupt.h>
#include "uart.h"
#include "relays.h"
#include "messages.h"
#include "rprintf.h"
#include "uart_protocol.h"

#define SIM900_OK_ANS "\r\nOK\r\n"


struct SMS_STRUCT
{
	uint8_t	  number[16];	
	uint8_t     date[11];
	uint8_t     time[11];
	uint8_t      msg[100];
};


typedef union
{
	uint8_t incoming_call_nr[16];
	struct SMS_STRUCT received_sms;
} sim900_info_union;


struct sterownik_opcje
{
	bool auto_ans;
	bool auto_hangup;
	
	uint8_t pin_number[9];
	uint8_t pass[10];
	
	bool out_state[12];
	
	bool allow_alarm1;
	bool allow_alarm2;
	
	bool power_on_sms;
	
	uint8_t main_num[13];
	uint8_t power_on_info[2];
};


typedef struct  
{
	uint8_t last_recv_chr;
	uint8_t recv_buffer[220];
	uint8_t send_buffer[181]; 
	uint8_t buffer_element;	
	sim900_info_union info;
	struct sterownik_opcje opcje;	
} SIM900_struct;





typedef enum {SIM_NOT_INSERTED,READY, PIN_ENTER, PUK_ENTER, PIN2_ENTER, PUK2_ENTER, ERROR} sim_state;




#define delay_ms(x) _delay_ms(x)
#define sim900_string(x) usart_gsm_string(x)
#define sim900_char(x) usart_gsm_char(x)

//Sprawdza stan pinu STATUSOWEGO SIM900
#define SIM900_STATUS_PIN (PIND & (1<<1))

#define SIM900_DTR(x)  ((x==1)?(PORTD &= (~(1<<4))):((PORTD |= 1<<4)))

//Inicjalizowanie pinw moduu SIM900
void sim900_pin_config(void);

//Resetuje modu SIM900
void sim900_reset(void);

//Podaje impuls POWER_EN mode 0: impuls wyczenia, 1: impuls zaczenia
//Zwraca: Operacja wykonana 0: niepoprawnie 1: poprawnie
bool sim900_power_switch(bool mode);

//Uruchamia funkcj automatycznie dostrajajc uart
bool sim900_autobaud(void);

//Konfiguracja moduu SIM900, preconfig 1-> wyczenie echa, 0-> dokonanie gwnej konfiguracji
bool sim900_configure(bool pre_config);

//Sterowanie portami GPIO moduu SIM900
//mode: 0 set gpio keyboard or output; 1 read gpio state (pin must be set to input)
//gpio_number: 1-12
//function: 0 input, 1 output, 2 keypad
//state: 0 set low level, 1 set high level
bool sim900_gpio_exec(bool mode, uint8_t gpio_number, uint8_t function, bool state);

//Funcja wysya do moduu komend AT okrelon acuchem command
//oczekuje na odpowied zawart w acuchu wait_answ przez "max_retr" cykli (wraz z retransmisj)
//w przypadku powodzenia zwraca 1
bool SIM900_send_command(uint8_t * command, uint8_t * wait_answ, uint8_t max_retr);

//Funkcja oczekiwania na odpowied
//ans: acuch znakw na ktry czekamy
//time_ms: czas po ktrym nastpuje timeout
//zwraca 1 w przypadku otrzymania komendy na ktr oczekujemy
bool wait_ans( uint8_t * ans, uint16_t time_ms);

//Funkcja sprawdzajca stan karty SIM
sim_state sim900_check_pin(void);
//Funkcja podajca PIN do karty SIM. Gdy modu odpowie 'OK' funkcja zwraca 1
bool sim900_enter_pin(uint8_t * pin_number);

//=========================Poczenia i SMS=========================
//Funckja odbierajca przychodzce poczenie
void sim900_call_ans(void);

//Funckja odrzucajca przychodzce poczenie
void sim900_call_refuse(void);

//Funkja wysyania sms, jako argumenty przyjmuje numer i wiadomo
//jeeli wysano zwraca 1
bool sim900_send_sms(uint8_t * number, uint8_t * msg);
//Kasowanie wszystkich smsow
bool delete_all_sms();
//Funkcja czyszczca bufor UART
void SIM900_buffer_erase(void);




#endif