#include <avr/io.h>
#include <avr/interrupt.h>
#include <stdbool.h>
#include "rprintf.h"
#include "types.h"
#include "relays.h"
#include "timers.h"
#include "uart.h"
#include "uart_protocol.h"
#include "sim900.h"
#include "messages.h"
#include "gsm_commands.h"
#include "pamiec_konfiguracji.h"
#include "adc.h"
#include "sms_command.h"
#include <avr/wdt.h>

uart_protocol_pack usart_frames;
uart_protocol sending_frame;
uart_protocol receiving_frame;


volatile uint16_t eraser_cnt=0;

SIM900_struct sim900struct;

void error_func(void);
bool gsm_start_func(void);

int main(void)
{
	uint8_t i;
	bool retval;
	uint8_t err_cnt;
	
	usart_frames.sending = &sending_frame;
	usart_frames.receiving = &receiving_frame;
	
	//Brak automatycznego odbierania pocze
	sim900struct.opcje.auto_ans=0;
	//Ukatywnione automatyczne odrzucanie poczenia
	sim900struct.opcje.auto_hangup=1;
	//Ustalenie kodu PIN
	czytaj_element(pin,0,sim900struct.opcje.pin_number,8);
	//Wczytania hasa:
	czytaj_element(password,0,sim900struct.opcje.pass,12);
	//Zezwolenie na alarm
	sim900struct.opcje.allow_alarm1=1;
	sim900struct.opcje.allow_alarm2=1;
	//Brak POWER_ON_SMS
	sim900struct.opcje.power_on_sms=1;
	//Odczyt numeru gwnego
	czytaj_element(main_number,0,sim900struct.opcje.main_num,12);
	sim900struct.opcje.main_num[12]=0;
	//Start info ?
	czytaj_element(start_info,0,sim900struct.opcje.power_on_info,1);
	
	

	
	
	for (i=0;i<12;i++)
		sim900struct.opcje.out_state[i]=0;
	
	ports_init(); //inicjalizacja portw mikrokontrolera 
	led_timer_init(1); //uruchomienie timera sterujcego sygnalizacj
	usart_usb_init(51);	//USB init baud 9600
	usart_gsm_init(3);	//GSM init baud 9600
	config_adc(1); //Uruchomienie ADC tryb single mode
	config_pwm();  //Uruchomienie PWM
	
	
	WDTCR |= (1<<WDCE) | (1<<WDE);
    wdt_enable(WDTO_2S);
	wdt_reset();
	
	
	if (S1_pressed)
	{
		led3_on;
		
		while(1)
		{
			wdt_reset();
			if (S1_pressed)
				PORTD |= (1<<6);
			else
				PORTD &= (~(1<<6));
		}
	}
	debug_txt(SYSTEM_START);
	
	
	

	
	#ifdef debug
		debug_eeprom();
	#endif

	while (gsm_start_func()!=1) wdt_reset();
	rprintfInit(usart_usb_char);
	
	sim900struct.buffer_element=0;
	sim900struct.recv_buffer[0]=0;
	
	
	if (sim900struct.opcje.power_on_info[0])
	{
		debug_txt(START_INFO);		
		err_cnt=5;
		
		do 
		{
			wdt_reset();
			retval=sim900_send_sms(sim900struct.opcje.main_num,START_INFO_SMS);
			if (retval == 0) 
			{
				_delay_ms(500);
				wdt_reset();
				_delay_ms(500);
				wdt_reset();
				_delay_ms(500);
				wdt_reset();
				_delay_ms(500);
				wdt_reset();
				debug_txt(WAIT_FOR_READY);										
			}				
			err_cnt--;
			
		} while ((retval == 0) && (err_cnt));
		
	}
	
		
	while(1)
	{
		wdt_reset();
		uart_protocol_exec_action(&usart_frames);
		gsm_command_verify(&sim900struct);
		
		
		if ((is_ALERT1) && (sim900struct.opcje.allow_alarm1))
		{
			sim900struct.opcje.allow_alarm1=0;
			err_cnt=3;
			do 
			{
				wdt_reset();
				alarm_msg(sim900struct.send_buffer);
				retval=sim900_send_sms(sim900struct.opcje.main_num,sim900struct.send_buffer);
				err_cnt--;
			
			} while ((retval == 0) && (err_cnt));	
		}
		
		
		wdt_reset();
		if ((is_ALERT2) && (sim900struct.opcje.allow_alarm2))
		{
			sim900struct.opcje.allow_alarm2=0;
			err_cnt=3;
			do 
			{
				wdt_reset();
				alarm_msg(sim900struct.send_buffer);
				retval=sim900_send_sms(sim900struct.opcje.main_num,sim900struct.send_buffer);
				err_cnt--;
			
			} while ((retval == 0) && (err_cnt));	
		}
		
		
		if (!SIM900_STATUS_PIN) //Modu z jakich przyczyn si wyczy -> dokonaj ponownego rozruchu systemu
			gsm_start_func();	
			
		wdt_reset();	
	}	
}


bool gsm_start_func(void)
{
	uint8_t i;
	sim_state stan_sim;
	
	cli();
	wdt_reset();
	led1_on;
	led2_on;
	led3_on;
	
	
	SIM900_buffer_erase();
	sim900_pin_config();
	sim900_reset();
	

	//Upewnienie si, e modu GSM jest wyczony
	if (SIM900_STATUS_PIN)
	{
		debug_txt("Wylaczanie GSM");
		wdt_reset();
		i=10;
		while(SIM900_STATUS_PIN)
		{
			sim900_string("AT+CPOWD=0\r");
			wdt_reset();
			_delay_ms(500);
			wdt_reset();
			i--;
			if (i==0) break;
		}	
	}
	
	wdt_reset();
	_delay_ms(500);
	wdt_reset();
	_delay_ms(500);
	
	if (sim900_power_switch(1) == 0)  //Uruchom modu GSM
	{
		debug_txt("Sim 900 start error");
	}
	else
	{
		debug_txt("Sim 900 start ok");
	}	
	
	wdt_reset();
	_delay_ms(200);
	
	i=20;
	while (sim900_autobaud()==0)
	{
		debug_txt("Autobaud ERROR");
		i--;
		wdt_reset();
		if (i==0) return 0;
	}	 
	
	debug_txt("Autobaud OK");
	
	led3_off;
	
	sei();
	
	//Dokonanie konfiguracji podstawowych parametrw moduu
	sim900_configure(1);
	
	wdt_reset();
	stan_sim = ERROR;
	i=10;
	while(stan_sim == ERROR)
	{
		wdt_reset();
		stan_sim = sim900_check_pin();
	
		if (stan_sim == READY)
		{
			debug_txt("SIM gotowa");	
		} 
		else if (stan_sim == PIN_ENTER)
		{	
			debug_txt("Podaj PIN");		
			if (sim900_enter_pin(sim900struct.opcje.pin_number)==0)
			{
				debug_txt("Zly PIN. Wstrzymanie systemu");
				error_func();
			}
			stan_sim = sim900_check_pin();
		}
		else if (stan_sim == SIM_NOT_INSERTED)
		{
			debug_txt("Brak SIM");
			error_func();
		}
		else
		{
			debug_txt("Inny wyjatek");
		}
		
		i--;
		wdt_reset();
		if (i==0) return 0;
	}			
	
	
	if (stan_sim != READY)
		return 0;
		
	led1_off;
	
	sim900_configure(0);	
	
	debug_txt("SIM gotowa");
	
	led2_off;
	
	return 1;
}


void error_func(void)
{
	
	
	while(1)
	{
		wdt_reset();
		led1_on;
		led2_on;
		led3_on;
		uart_protocol_exec_action(&usart_frames);
	}
}


ISR(TIMER0_OVF_vect)
{
	eraser_cnt++;
	
	if (eraser_cnt >= 10000)
	{
		eraser_cnt=0;
		sim900struct.recv_buffer[0]=0;
		sim900struct.buffer_element=0;
	}
	
	
	if (eraser_cnt >=100) led2_off;
	
}

ISR(USART1_RX_vect)
{
	wdt_reset();
	uint8_t chr=UDR1;
	protocol_function_recv_int(chr,&usart_frames);
}

