#include "uart_protocol.h"

#ifdef debug
	u8 debug_string[40];
#endif

//------Funkca sprawdzajca czy przychodzce dane s zgodne z protokoem-----
//Funkcja powinna by wywoywana przez przerwanie od nadchodzcych danych do Uartu
//------------------------Format protokou-----------------------------------
//[Sowo startu 0x3C 0x3C] (2B)
//[Adres nadawcy] (1B)
//[Adres odbiorcy] (1B)
//[Dugos danych] (1B)
//[Dane] (xB)
//[CRC] (2B)
void protocol_function_recv_int(uint8_t received_char, uart_protocol_pack * usart_frames)
{
	static uint16_t calc_crc;
	static uint8_t state_of_sync;
	static uint8_t i;
	static uint8_t data_len;

	uart_protocol * odebrana_ramka;

	odebrana_ramka = usart_frames->receiving;

	odebrana_ramka->ok_flag=0;


	if (received_char == 0x0D) //Carriage return -> ACK
	{
		odebrana_ramka->is_ack=1;
	}



	switch (state_of_sync)
	{
		case 0: //Sowa startu
		case 1: //Sowa startu
		
			if (received_char == start_word)
			{
				state_of_sync++;
			}
			else
				state_of_sync = 0;

			break;

		case 2: // Adres nadawcy
			 
			if (received_char <= max_source_address)
			{
				calc_crc = UpdateCRC(received_char,0xFFFF);
				odebrana_ramka->source=received_char;
				state_of_sync++;
			}
			else
				state_of_sync =0;

			break;
		case 3: //Adres odbiorcy
			if (received_char <=max_destination_addres)
			{
				calc_crc = UpdateCRC(received_char,calc_crc);
				odebrana_ramka->dest=received_char;
				state_of_sync++;
			} else
				state_of_sync=0;
			break;

		case 4: //Request
				calc_crc = UpdateCRC(received_char,calc_crc);
				odebrana_ramka->reqest=received_char;
				state_of_sync++;

			break;
		
		case 5: //Request2
				calc_crc = UpdateCRC(received_char,calc_crc);
				odebrana_ramka->request2=received_char;
				state_of_sync++;

			break;

		case 6: //Dugosc danych
			
			if (received_char <= max_data_len)
			{
				calc_crc = UpdateCRC(received_char,calc_crc);
				data_len = received_char;
				odebrana_ramka->len=data_len;
				i=0;
				state_of_sync++;
			}
			else
				state_of_sync=0;

			break;

		case 7: //Pakiet danych
			odebrana_ramka->pakiet_danych[i++] = received_char;
			calc_crc = UpdateCRC(received_char,calc_crc);

			if (i == data_len)
				state_of_sync++;
			else if (i > data_len)
				state_of_sync = 0;

			break;

		case 8://CRC Bajt 1
			if (received_char == ((calc_crc >> 8)&0xFF))
				state_of_sync++;
			else
				state_of_sync=0;

			break;
		case 9://CRC Bajt 2
			if (received_char == ((calc_crc)&0xFF)) // Odebrano popran ramk
			{
				odebrana_ramka->ok_flag=1;
				state_of_sync=0;
			}
			else
			{
				state_of_sync=0;
			}
			break;

		default:
				state_of_sync=0;
				break;
	}
}


void send_frame(uart_protocol * frame_to_send)
{
	uint8_t cnt=0;
	uint16_t calc_crc=0;
	
	send_data(0x3C); //Sowa startu
	send_data(0x3C);
	send_data(frame_to_send->source);
	send_data(frame_to_send->dest);
	send_data(frame_to_send->reqest);
	send_data(frame_to_send->request2);
	send_data(frame_to_send->len);
	
	calc_crc = UpdateCRC(frame_to_send->source,0xFFFF);
	calc_crc = UpdateCRC(frame_to_send->dest,calc_crc);
	calc_crc = UpdateCRC(frame_to_send->reqest,calc_crc);
	calc_crc = UpdateCRC(frame_to_send->request2,calc_crc);
	calc_crc = UpdateCRC(frame_to_send->len,calc_crc);


	for (cnt=0;cnt<frame_to_send->len;cnt++)
	{
		send_data(frame_to_send->pakiet_danych[cnt]);
		calc_crc = UpdateCRC(frame_to_send->pakiet_danych[cnt],calc_crc);
	}

	send_data((calc_crc >>8) & 0xFF);
	send_data((calc_crc ) & 0xFF);
}



uint16_t UpdateCRC(uint8_t byte, uint16_t crc)
{
	byte ^= (uint8_t)(crc & 0x00ff);
	byte ^= (uint8_t)(byte << 4);
	return ((((uint16_t)byte << 8) | (uint8_t)(crc >> 8)) ^ (uint8_t)(byte >> 4) ^ ((uint16_t)byte << 3));
}

void send_protocol_string(uart_protocol_pack * uart_frames, uint8_t * message, bool wait_for_ack)
{
	uint8_t cnt_retr = num_of_retransmission;
	uart_protocol * uart_frame;

	uart_frame = uart_frames->sending;
	cnt_retr++;

	do
	{
		cnt_retr--;

		if (cnt_retr == 0)
			break;


		uart_frame->source=0x00;
		uart_frame->dest=0x00;
		uart_frame->len=strlen(message);
		uart_frame->reqest=send_string_msg;
		uart_frame->request2=0;
		strcpy(uart_frame->pakiet_danych,message);
		send_frame(uart_frame);
		uart_frames->receiving->is_ack=0;

		if (wait_for_ack == 1)
		{
			delay_ms(5);

			if ((uart_frames->receiving->is_ack)==0)
				delay_ms(300);

			if ((uart_frames->receiving->is_ack)==1)
				break;

		}

	} while (wait_for_ack == 1);
	
	delay_ms(100);
}


void send_protocol_frame(uart_protocol_pack * uart_frames,uint8_t source_adr, uint8_t dest_adr, request_type request , uint8_t request2, uint8_t * message, uint8_t msg_len, bool wait_for_ack, uint8_t retr_cnt)
{
	uint8_t cnt_retr = retr_cnt;
	uart_protocol * uart_frame;

	uart_frame = uart_frames->sending;
	cnt_retr++;
	uint16_t i;

	do
	{
		if (retr_cnt != 0)
			cnt_retr--;

		if (cnt_retr == 0)
			break;


		uart_frame->source=source_adr;
		uart_frame->dest=dest_adr;
		uart_frame->len=msg_len;
		uart_frame->reqest=request;
		uart_frame->request2=request2;
		
		for (i=0;i<msg_len;i++)
		{
			uart_frame->pakiet_danych[i] = message[i];
		}		

		send_frame(uart_frame);
		uart_frames->receiving->is_ack=0;

		if (wait_for_ack == 1)
		{
			delay_ms(5);

			if ((uart_frames->receiving->is_ack)==0)
				delay_ms(200);

			if ((uart_frames->receiving->is_ack)==1)
				break;

		}

	} while (wait_for_ack == 1);
}

void send_ack(void)
{
	send_data(0x0D);
}