
#ifndef H_uart_protocol
#define H_uart_protocol

#include <stdbool.h>
#include <string.h>
#include <util/delay.h>
#include "uart.h"
#include "relays.h"


#define num_of_retransmission 5;
#define send_data(x) usart_usb_char(x)
#define usart_char(x,y) (usart_usb_char(y))
#define delay_ms(x) _delay_ms(x)

typedef enum
{
        send_string_msg = 0x01,
        
        read_digin_name = 0x02,
        set_digin_name  = 0x03,
        send_digin_name = 0x04,

        read_analogin_name = 0x05,
        set_analogin_name = 0x06,
        send_analogin_name = 0x07,

        read_digout_name = 0x08,
        set_digout_name = 0x09,
        send_digout_name = 0x0A,
		
		read_alarm_name = 0x0B,
        set_alarm_name = 0x0C,
        send_alarm_name = 0x0D,

        read_pwm_name = 0x0E,
        set_pwm_name = 0x0F,
        send_pwm_name = 0x10,
		
		read_main_num = 0x11,
        set_main_num = 0x12,
        send_main_num = 0x13,
		
	
        set_pin = 0x14,	
		set_pass = 0x15,
 

        //get_digout_state = 0xA4,
        set_digout_state = 0xA5,
        set_pwm_state = 0xA6,
		
		
		tst = 0xFF
		
} request_type ;




typedef struct
{
	bool is_ack;
	bool ok_flag;
	uint8_t source;
	uint8_t dest;
	uint8_t 	len;
	request_type reqest;
	uint8_t request2;
	uint8_t  pakiet_danych[40+1];
} uart_protocol;


typedef struct
{
	uart_protocol * sending;
	uart_protocol * receiving;
}   uart_protocol_pack;


//------------------------------Definicje parametrw ----------------------------



//#define send_data(uart,chr) (usart_char(uart,chr))
//#define send_data(uart,chr) (UsbSendData(&##chr,1))
#define start_word 0x3c

#define max_source_address 0xFF
#define max_destination_addres 0xFF
#define max_data_len 50


extern uart_protocol_pack usart_frames;
#define debug_txt(x) send_protocol_string(&usart_frames,x,1)

//------Funkca sprawdzajca czy przychodzce dane s zgodne z protokoem-----
//Funkcja powinna by wywoywana przez przerwanie od nadchodzcych danych do Uartu
//------------------------Format protokou-----------------------------------
//[Sowo startu 0x3C 0x3C] (2B)
//[Adres nadawcy] (1B)
//[Adres odbiorcy] (1B)
//[danie] (1B)
//[Dugos danych] (1B)
//[Dane] (xB)
//[CRC] (2B)

//Wysya ramk danych poprzez UART
void send_frame(uart_protocol * frame_to_send);

//funkcja prztwarzajca odbierane dane
void protocol_function_recv_int(uint8_t received_char, uart_protocol_pack * usart_frames);
uint16_t UpdateCRC(uint8_t byte, uint16_t crc);

//Wysyanie stringa w poprzez protok, adresy nadawcy i odbiorcy ustawione na 0
void send_protocol_string(uart_protocol_pack * uart_frames, uint8_t * message, bool wait_for_ack);
//Wysyanie danych w protokoem uart
void send_protocol_frame(uart_protocol_pack * uart_frames,uint8_t source_adr, uint8_t dest_adr, request_type, uint8_t request2, uint8_t * message, uint8_t msg_len, bool wait_for_ack,uint8_t retr_cnt);
//wysyanie potwierdzenia odbioru ramki
void send_ack(void);

#endif
